<?php

/*
CLASSE COPIADA COM MODIFICAÇÕES DA DEPENDENCIA 'TEKNISA/LIBRARIES' VERSÃO: '3.0.24'. NAMESPACE ORIGINAL 'Teknisa\Libs\Util'
*/

namespace PME\Auth;

use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\Session\Session;

use PME\Util\Messages;
use PME\Auth\Exception\AccessException;

class RequestListenerOAuth extends Listener {

    public function __construct(Kernel $kernel, OAuthCustom $oauth, Session $sessionManager) {
        $this->httpKernel       = $kernel;
        $this->oauth            = $oauth;
        $this->sessionManager   = $sessionManager;
        $this->publicRoutes     = $this->readPublicRoutes();
    }

    const           AUTHORIZATION_REQUEST_HEADER = 'oAuthToken';
    const           OAUTH_KEEP_CONNECTED = "OAuthKeepConnected";
    private const   DEFAULT_TOKEN_VALUE = null;

    const           KEEP_CONNECTED_POSITIVE = 'Yes';
    const           PUBLIC_ROUTES_PATH = '/../../config/publicRoutes.json';

    public function preDispatch(DTO\Request $request) {
        $apacheHeaders = apache_request_headers();
        // ini_set('session.use_cookies', 0);
        $route = $request->getRoutePath();
        $isRoutePublic = $this->isRoutePublic($route);

        if ($isRoutePublic) {
            return;
        }
        
        $token = $this->getRequestToken($apacheHeaders);
      
        if (!is_null($token)) {
            $service = $this->checkAccessAndSetSession($token);
        } else {
            throw new AccessException();
        }
    }

    public function checkAccessAndSetSession($token) {
        $secretId = null;
        $apacheHeaders = apache_request_headers();
        $keepConn = $apacheHeaders[self::OAUTH_KEEP_CONNECTED];
        $keepConn = $keepConn == self::KEEP_CONNECTED_POSITIVE ? true : false;
        $access = $this->oauth->checkAccess($token, $secretId, array(), $keepConn);
        if (!empty($access)) {
            if($this->sessionManager->getId() != $access['sessionId']) {
                $this->sessionManager->start();
                $this->sessionManager->destroy();
                $this->sessionManager->setId($access['sessionId']);
            }
            $this->sessionManager->start();
            if(!$this->sessionManager->has("NRORG")) {
                $this->sessionManager->set('NRORG', $access['service']->getNrOrg());
                $this->sessionManager->set('CDOPERADOR', $access['service']->getCdOperator());
            }

            if(!$this->sessionManager->has("ORGANIZATION_ID")) {
                $this->sessionManager->set('ORGANIZATION_ID', $access['service']->getNrOrg());
                $this->sessionManager->set('USER_ID', $access['service']->getCdOperator());
            }
        }
    }

    private function readPublicRoutes() {
        $publicRoutesPath = $this->getPublicRoutesPath();
        $urls = file_get_contents($publicRoutesPath);
        return json_decode($urls, true);
    }

    private function getPublicRoutesPath() {
        return __DIR__ . self::PUBLIC_ROUTES_PATH;
    }

    private function isRoutePublic($route) {
        return in_array($route, $this->publicRoutes);
    }

    // Busca pelo nome do 'Header' do token, com o case original ou em lowercase
    private function getRequestToken($headers) {
        $token = isset($headers[self::AUTHORIZATION_REQUEST_HEADER]) ? $headers[self::AUTHORIZATION_REQUEST_HEADER] : self::DEFAULT_TOKEN_VALUE;
        if (is_null($token)) {

            $tokenHeaderNameInLowercase = strtolower(self::AUTHORIZATION_REQUEST_HEADER);
            $token = isset($headers[$tokenHeaderNameInLowercase]) ? $headers[$tokenHeaderNameInLowercase] : self::DEFAULT_TOKEN_VALUE;
        }
        return $token;
    }

}